@@ -6,7 +6,11 @@
  */
 package org.mule.module.launcher;
 
+import static com.google.common.base.Optional.absent;
+import static org.apache.commons.lang.StringUtils.removeEndIgnoreCase;
+import static org.mule.module.launcher.DeploymentPropertiesUtils.resolveDeploymentProperties;
 import static org.mule.util.SplashScreen.miniSplash;
+
 import org.mule.config.i18n.CoreMessages;
 import org.mule.config.i18n.MessageFactory;
 import org.mule.module.launcher.application.NullDeploymentListener;
@@ -14,7 +18,6 @@ import org.mule.module.launcher.artifact.Artifact;
 import org.mule.module.launcher.artifact.ArtifactFactory;
 import org.mule.module.launcher.util.ObservableList;
 import org.mule.util.CollectionUtils;
-import org.mule.util.StringUtils;
 
 import java.io.File;
 import java.io.IOException;
@@ -23,9 +26,9 @@ import java.net.URL;
 import java.util.Collection;
 import java.util.HashMap;
 import java.util.Map;
-import java.util.concurrent.TimeUnit;
-import java.util.concurrent.locks.ReentrantLock;
+import java.util.Properties;
 
+import com.google.common.base.Optional;
 import org.apache.commons.beanutils.BeanPropertyValueEqualsPredicate;
 import org.apache.commons.beanutils.BeanToPropertyValueTransformer;
 import org.apache.commons.logging.Log;
@@ -42,40 +45,39 @@ public class DefaultArchiveDeployer<T extends Artifact> implements ArchiveDeploy
 
     public static final String ARTIFACT_NAME_PROPERTY = "artifactName";
     public static final String ZIP_FILE_SUFFIX = ".zip";
-    public static final String ANOTHER_DEPLOYMENT_OPERATION_IS_IN_PROGRESS = "Another deployment operation is in progress";
-    public static final String INSTALL_OPERATION_HAS_BEEN_INTERRUPTED = "Install operation has been interrupted";
     private static final Log logger = LogFactory.getLog(DefaultArchiveDeployer.class);
 
     private final ArtifactDeployer<T> deployer;
     private final ArtifactArchiveInstaller artifactArchiveInstaller;
-    private final ReentrantLock deploymentLock;
     private final Map<String, ZombieFile> artifactZombieMap = new HashMap<String, ZombieFile>();
     private final File artifactDir;
     private final ObservableList<T> artifacts;
+    private final ArtifactDeploymentTemplate deploymentTemplate;
     private ArtifactFactory<T> artifactFactory;
     private DeploymentListener deploymentListener = new NullDeploymentListener();
 
-
-    public DefaultArchiveDeployer(final ArtifactDeployer deployer, final ArtifactFactory artifactFactory, final ObservableList<T> artifacts, final ReentrantLock lock)
+    public DefaultArchiveDeployer(final ArtifactDeployer deployer, final ArtifactFactory artifactFactory, final ObservableList<T> artifacts,
+                                  ArtifactDeploymentTemplate deploymentTemplate)
     {
         this.deployer = deployer;
         this.artifactFactory = artifactFactory;
         this.artifacts = artifacts;
-        this.deploymentLock = lock;
+        this.deploymentTemplate = deploymentTemplate;
         this.artifactDir = artifactFactory.getArtifactDir();
         this.artifactArchiveInstaller = new ArtifactArchiveInstaller(artifactDir);
     }
 
-    public T deployPackagedArtifact(String zip) throws DeploymentException
+    @Override
+    public T deployPackagedArtifact(String zip, Optional<Properties> deploymentProperties) throws DeploymentException
     {
         URL url;
         File artifactZip;
         try
         {
-            final String artifactName = StringUtils.removeEnd(zip, ZIP_FILE_SUFFIX);
+            final String artifactName = removeEndIgnoreCase(zip, ZIP_FILE_SUFFIX);
             artifactZip = new File(artifactDir, zip);
             url = artifactZip.toURI().toURL();
-            return deployPackagedArtifact(url, artifactName);
+            return deployPackagedArtifact(url, artifactName, deploymentProperties);
         }
         catch (DeploymentException e)
         {
@@ -87,33 +89,51 @@ public class DefaultArchiveDeployer<T extends Artifact> implements ArchiveDeploy
         }
     }
 
+    @Override
     public T deployExplodedArtifact(String artifactDir) throws DeploymentException
     {
-        String artifactName = artifactDir;
+        if (!isUpdatedZombieArtifact(artifactDir))
+        {
+            return null;
+        }
+
+        return deployExplodedApp(artifactDir);
+    }
+
+    @Override
+    public boolean isUpdatedZombieArtifact(String artifactName)
+    {
         @SuppressWarnings("rawtypes")
         Collection<String> deployedAppNames = CollectionUtils.collect(artifacts, new BeanToPropertyValueTransformer(ARTIFACT_NAME_PROPERTY));
 
         if (deployedAppNames.contains(artifactName) && (!artifactZombieMap.containsKey(artifactName)))
         {
-            return null;
+            return false;
         }
 
         ZombieFile zombieFile = artifactZombieMap.get(artifactName);
 
         if ((zombieFile != null) && (!zombieFile.updatedZombieApp()))
         {
-            return null;
+            return false;
         }
-
-        return deployExplodedApp(artifactName);
+        return true;
     }
 
+    @Override
     public void undeployArtifact(String artifactId)
     {
         ZombieFile zombieFile = artifactZombieMap.get(artifactId);
+
         if ((zombieFile != null))
         {
-            return;
+            if (zombieFile.exists()) {
+                return;
+            }
+            else
+            {
+                artifactZombieMap.remove(artifactId);
+            }
         }
 
         T artifact = (T) CollectionUtils.find(artifacts, new BeanPropertyValueEqualsPredicate(ARTIFACT_NAME_PROPERTY, artifactId));
@@ -126,21 +146,39 @@ public class DefaultArchiveDeployer<T extends Artifact> implements ArchiveDeploy
         return artifactFactory.getArtifactDir();
     }
 
-    public T deployPackagedArtifact(URL artifactAchivedUrl) throws DeploymentException
+    public T deployPackagedArtifact(URL artifactAchivedUrl, Optional<Properties> deploymentProperties) throws DeploymentException
     {
         T artifact;
-
+        String artifactName;
         try
         {
             try
             {
-                artifact = guardedInstallFrom(artifactAchivedUrl);
+                artifactName = artifactArchiveInstaller.installArtifact(artifactAchivedUrl);
+            }
+            catch (Throwable t)
+            {
+                File artifactArchive = new File(artifactAchivedUrl.toURI());
+                artifactName = removeEndIgnoreCase(artifactArchive.getName(), ".zip");
+
+                // error text has been created by the deployer already
+                logDeploymentFailure(t, artifactName);
+
+                addZombieFile(artifactName, artifactArchive);
+
+                deploymentListener.onDeploymentFailure(artifactName, t);
+
+                throw t;
+            }
+
+            try
+            {
+                artifact = artifactFactory.createArtifact(artifactName, deploymentProperties);
                 trackArtifact(artifact);
             }
             catch (Throwable t)
             {
                 File artifactArchive = new File(artifactAchivedUrl.toURI());
-                String artifactName = StringUtils.removeEnd(artifactArchive.getName(), ZIP_FILE_SUFFIX);
 
                 // error text has been created by the deployer already
                 logDeploymentFailure(t, artifactName);
@@ -167,13 +205,14 @@ public class DefaultArchiveDeployer<T extends Artifact> implements ArchiveDeploy
             throw new DeploymentException(MessageFactory.createStaticMessage(msg), t);
         }
     }
-
+  
     private void logDeploymentFailure(Throwable t, String artifactName)
     {
         final String msg = miniSplash(String.format("Failed to deploy artifact '%s', see below", artifactName));
         logger.error(msg, t);
     }
 
+    @Override
     public Map<URL, Long> getArtifactsZombieMap()
     {
         Map<URL, Long> result = new HashMap<URL, Long>();
@@ -186,6 +225,7 @@ public class DefaultArchiveDeployer<T extends Artifact> implements ArchiveDeploy
         return result;
     }
 
+    @Override
     public void setArtifactFactory(final ArtifactFactory<T> artifactFactory)
     {
         this.artifactFactory = artifactFactory;
@@ -197,11 +237,6 @@ public class DefaultArchiveDeployer<T extends Artifact> implements ArchiveDeploy
         logRequestToUndeployArtifact(artifact);
         try
         {
-            if (!deploymentLock.tryLock(0, TimeUnit.SECONDS))
-            {
-                return;
-            }
-
             deploymentListener.onUndeploymentStart(artifact.getArtifactName());
             deployer.undeploy(artifact);
             deploymentListener.onUndeploymentSuccess(artifact.getArtifactName());
@@ -211,17 +246,6 @@ public class DefaultArchiveDeployer<T extends Artifact> implements ArchiveDeploy
             deploymentListener.onUndeploymentFailure(artifact.getArtifactName(), e);
             throw e;
         }
-        catch (InterruptedException e)
-        {
-            Thread.currentThread().interrupt();
-        }
-        finally
-        {
-            if (deploymentLock.isHeldByCurrentThread())
-            {
-                deploymentLock.unlock();
-            }
-        }
     }
 
     ArtifactDeployer getDeployer()
@@ -229,12 +253,13 @@ public class DefaultArchiveDeployer<T extends Artifact> implements ArchiveDeploy
         return deployer;
     }
 
+    @Override
     public void setDeploymentListener(CompositeDeploymentListener deploymentListener)
     {
         this.deploymentListener = deploymentListener;
     }
 
-    private T deployPackagedArtifact(final URL artifactUrl, String artifactName) throws IOException
+    private T deployPackagedArtifact(final URL artifactUrl, String artifactName, Optional<Properties> deploymentProperties) throws IOException
     {
         ZombieFile zombieFile = artifactZombieMap.get(artifactName);
         if (zombieFile != null)
@@ -250,10 +275,13 @@ public class DefaultArchiveDeployer<T extends Artifact> implements ArchiveDeploy
         T artifact = (T) CollectionUtils.find(artifacts, new BeanPropertyValueEqualsPredicate(ARTIFACT_NAME_PROPERTY, artifactName));
         if (artifact != null)
         {
+            deploymentTemplate.preRedeploy(artifact);
             undeployArtifact(artifactName);
         }
 
-        return deployPackagedArtifact(artifactUrl);
+        T deployedAtifact = deployPackagedArtifact(artifactUrl, deploymentProperties);
+        deploymentTemplate.postRedeploy(deployedAtifact);
+        return deployedAtifact;
     }
 
     private T deployExplodedApp(String addedApp) throws DeploymentException
@@ -298,59 +326,11 @@ public class DefaultArchiveDeployer<T extends Artifact> implements ArchiveDeploy
         return artifact;
     }
 
-
-    private void guardedDeploy(T artifact)
-    {
-        try
-        {
-            if (!deploymentLock.tryLock(0, TimeUnit.SECONDS))
-            {
-                return;
-            }
-            deployer.deploy(artifact);
-        }
-        catch (InterruptedException e)
-        {
-            Thread.currentThread().interrupt();
-        }
-        finally
-        {
-            if (deploymentLock.isHeldByCurrentThread())
-            {
-                deploymentLock.unlock();
-            }
-        }
-    }
-
+    @Override
     public void deployArtifact(T artifact) throws DeploymentException
     {
-        try
-        {
-            deploymentListener.onDeploymentStart(artifact.getArtifactName());
-            guardedDeploy(artifact);
-            artifactArchiveInstaller.createAnchorFile(artifact.getArtifactName());
-            deploymentListener.onDeploymentSuccess(artifact.getArtifactName());
-            artifactZombieMap.remove(artifact.getArtifactName());
-        }
-        catch (Throwable t)
-        {
-            // error text has been created by the deployer already
-            String msg = miniSplash(String.format("Failed to deploy artifact '%s', see below", artifact.getArtifactName()));
-            logger.error(msg, t);
-
-            addZombieApp(artifact);
-
-            deploymentListener.onDeploymentFailure(artifact.getArtifactName(), t);
-            if (t instanceof DeploymentException)
-            {
-                throw (DeploymentException) t;
-            }
-            else
-            {
-                msg = "Failed to deploy artifact: " + artifact.getArtifactName();
-                throw new DeploymentException(MessageFactory.createStaticMessage(msg), t);
-            }
-        }
+        Optional<Properties> properties = absent();
+        deployArtifact(artifact, properties);
     }
 
     private void addZombieApp(Artifact artifact)
@@ -400,12 +380,17 @@ public class DefaultArchiveDeployer<T extends Artifact> implements ArchiveDeploy
 
     private void trackArtifact(T artifact)
     {
-        T previousArtifact = findArtifact(artifact.getArtifactName());
-        artifacts.remove(previousArtifact);
+        preTrackArtifact(artifact);
 
         artifacts.add(artifact);
     }
 
+    public void preTrackArtifact(T artifact)
+    {
+        T previousArtifact = findArtifact(artifact.getArtifactName());
+        artifacts.remove(previousArtifact);
+    }
+
     private void undeploy(T artifact)
     {
         logRequestToUndeployArtifact(artifact);
@@ -414,9 +399,13 @@ public class DefaultArchiveDeployer<T extends Artifact> implements ArchiveDeploy
             deploymentListener.onUndeploymentStart(artifact.getArtifactName());
 
             artifacts.remove(artifact);
-            guardedUndeploy(artifact);
+
+            deployer.undeploy(artifact);
+            artifactArchiveInstaller.desinstallArtifact(artifact.getArtifactName());
 
             deploymentListener.onUndeploymentSuccess(artifact.getArtifactName());
+
+            logArtifactUndeployed(artifact);
         }
         catch (RuntimeException e)
         {
@@ -433,114 +422,27 @@ public class DefaultArchiveDeployer<T extends Artifact> implements ArchiveDeploy
         }
     }
 
-    private T guardedInstallFrom(URL artifactUrl) throws IOException
-    {
-        try
-        {
-            if (!deploymentLock.tryLock(0, TimeUnit.SECONDS))
-            {
-                throw new IOException(ANOTHER_DEPLOYMENT_OPERATION_IS_IN_PROGRESS);
-            }
-            return installFrom(artifactUrl);
-        }
-        catch (InterruptedException e)
-        {
-            Thread.currentThread().interrupt();
-            throw new IOException(INSTALL_OPERATION_HAS_BEEN_INTERRUPTED);
-        }
-        finally
-        {
-            if (deploymentLock.isHeldByCurrentThread())
-            {
-                deploymentLock.unlock();
-            }
-        }
-    }
-
-    private T installFrom(URL url) throws IOException
-    {
-        String artifactName = artifactArchiveInstaller.installArtifact(url);
-        return artifactFactory.createArtifact(artifactName);
-    }
-
-    private void guardedUndeploy(T artifact)
-    {
-        try
-        {
-            if (!deploymentLock.tryLock(0, TimeUnit.SECONDS))
-            {
-                return;
-            }
-
-            deployer.undeploy(artifact);
-            artifactArchiveInstaller.desinstallArtifact(artifact.getArtifactName());
-        }
-        catch (InterruptedException e)
-        {
-            Thread.currentThread().interrupt();
-        }
-        finally
-        {
-            if (deploymentLock.isHeldByCurrentThread())
-            {
-                deploymentLock.unlock();
-            }
-        }
-    }
-
-    public void redeploy(T artifact) throws DeploymentException
+    private void logArtifactUndeployed(T artifact)
     {
         if (logger.isInfoEnabled())
         {
-            logger.info(miniSplash(String.format("Redeploying artifact '%s'", artifact.getArtifactName())));
+            logger.info(miniSplash(String.format("Undeployed artifact '%s'", artifact.getArtifactName())));
         }
+    }
 
-        deploymentListener.onUndeploymentStart(artifact.getArtifactName());
-        try
-        {
-            deployer.undeploy(artifact);
-            deploymentListener.onUndeploymentSuccess(artifact.getArtifactName());
-        }
-        catch (Throwable e)
-        {
-            //TODO make the exception beter
-            deploymentListener.onUndeploymentFailure(artifact.getArtifactName(), e);
-        }
-
-        deploymentListener.onDeploymentStart(artifact.getArtifactName());
-        try
-        {
-            deployer.deploy(artifact);
-            artifactArchiveInstaller.createAnchorFile(artifact.getArtifactName());
-            deploymentListener.onDeploymentSuccess(artifact.getArtifactName());
-        }
-        catch (Throwable t)
-        {
-            try
-            {
-                logDeploymentFailure(t, artifact.getArtifactName());
-                if (t instanceof DeploymentException)
-                {
-                    throw (DeploymentException) t;
-                }
-                String msg = "Failed to deploy artifact: " + artifact.getArtifactName();
-                throw new DeploymentException(MessageFactory.createStaticMessage(msg), t);
-            }
-            finally
-            {
-                deploymentListener.onDeploymentFailure(artifact.getArtifactName(), t);
-            }
-        }
-
-        artifactZombieMap.remove(artifact.getArtifactName());
+    @Override
+    public void redeploy(T artifact) throws DeploymentException
+    {
+        Optional<Properties> properties = absent();
+        redeploy(artifact, properties);
     }
 
     private static class ZombieFile
     {
 
-        URL url;
-        Long originalTimestamp;
-        File file;
+        private URL url;
+        private Long originalTimestamp;
+        private File file;
 
         private ZombieFile(File file)
         {
@@ -565,5 +467,102 @@ public class DefaultArchiveDeployer<T extends Artifact> implements ArchiveDeploy
         {
             return originalTimestamp != file.lastModified();
         }
+
+        public boolean exists()
+        {
+            return file.exists();
+        }
+    }
+
+    @Override
+    public void redeploy(T artifact, Optional<Properties> deploymentProperties) throws DeploymentException
+    {
+        if (logger.isInfoEnabled())
+        {
+            logger.info(miniSplash(String.format("Redeploying artifact '%s'", artifact.getArtifactName())));
+        }
+
+        deploymentListener.onUndeploymentStart(artifact.getArtifactName());
+        try
+        {
+            deployer.undeploy(artifact);
+            deploymentListener.onUndeploymentSuccess(artifact.getArtifactName());
+        }
+        catch (Throwable e)
+        {
+            // TODO make the exception better
+            deploymentListener.onUndeploymentFailure(artifact.getArtifactName(), e);
+        }
+
+        deploymentListener.onDeploymentStart(artifact.getArtifactName());
+        try
+        {
+            artifact.setDeploymentProperties(resolveDeploymentProperties(artifact.getArtifactName(), deploymentProperties));
+            deployer.deploy(artifact);
+            artifactArchiveInstaller.createAnchorFile(artifact.getArtifactName());
+            deploymentListener.onDeploymentSuccess(artifact.getArtifactName());
+        }
+        catch (Throwable t)
+        {
+            try
+            {
+                logDeploymentFailure(t, artifact.getArtifactName());
+                addZombieApp(artifact);
+                if (t instanceof DeploymentException)
+                {
+                    throw (DeploymentException) t;
+                }
+                String msg = "Failed to deploy artifact: " + artifact.getArtifactName();
+                throw new DeploymentException(MessageFactory.createStaticMessage(msg), t);
+            }
+            finally
+            {
+                deploymentListener.onDeploymentFailure(artifact.getArtifactName(), t);
+            }
+        }
+
+        artifactZombieMap.remove(artifact.getArtifactName());
+    }
+
+    @Override
+    public void deployArtifact(T artifact, Optional<Properties> deploymentProperties) throws DeploymentException
+    {
+        try
+        {
+            // add to the list of known artifacts first to avoid deployment loop on failure
+            trackArtifact(artifact);
+
+            deploymentListener.onDeploymentStart(artifact.getArtifactName());
+            deployer.deploy(artifact);
+            artifactArchiveInstaller.createAnchorFile(artifact.getArtifactName());
+            deploymentListener.onDeploymentSuccess(artifact.getArtifactName());
+            artifactZombieMap.remove(artifact.getArtifactName());
+        }
+        catch (Throwable t)
+        {
+            // error text has been created by the deployer already
+            String msg = miniSplash(String.format("Failed to deploy artifact '%s', see below", artifact.getArtifactName()));
+            logger.error(msg, t);
+
+            addZombieApp(artifact);
+
+            deploymentListener.onDeploymentFailure(artifact.getArtifactName(), t);
+            if (t instanceof DeploymentException)
+            {
+                throw (DeploymentException) t;
+            }
+            else
+            {
+                msg = "Failed to deploy artifact: " + artifact.getArtifactName();
+                throw new DeploymentException(MessageFactory.createStaticMessage(msg), t);
+            }
+        }
+    }
+
+    @Override
+    public T deployPackagedArtifact(String zip) throws DeploymentException
+    {
+        Optional<Properties> properties = absent();
+        return deployPackagedArtifact(zip, properties);
     }
 }
