@@ -18,16 +18,20 @@ import org.mule.api.endpoint.ImmutableEndpoint;
 import org.mule.api.endpoint.InboundEndpoint;
 import org.mule.api.endpoint.OutboundEndpoint;
 import org.mule.api.lifecycle.InitialisationException;
+import org.mule.api.retry.RetryCallback;
+import org.mule.api.retry.RetryContext;
+import org.mule.api.transport.Connector;
 import org.mule.api.transport.ConnectorException;
 import org.mule.api.transport.DispatchException;
 import org.mule.api.transport.MessageReceiver;
 import org.mule.config.i18n.CoreMessages;
 import org.mule.config.i18n.MessageFactory;
 import org.mule.model.streaming.CallbackOutputStream;
-import org.mule.transport.AbstractConnector;
+import org.mule.retry.RetryPolicyExhaustedException;
 import org.mule.transport.ConnectException;
 import org.mule.transport.file.ExpressionFilenameParser;
 import org.mule.transport.file.FilenameParser;
+import org.mule.transport.nameable.AbstractInboundEndpointNameableConnector;
 import org.mule.util.ClassUtils;
 import org.mule.util.StringUtils;
 
@@ -44,7 +48,7 @@ import org.apache.commons.net.ftp.FTPFile;
 import org.apache.commons.pool.ObjectPool;
 import org.apache.commons.pool.impl.GenericObjectPool;
 
-public class FtpConnector extends AbstractConnector
+public class FtpConnector extends AbstractInboundEndpointNameableConnector
 {
 
     public static final String FTP = "ftp";
@@ -54,6 +58,7 @@ public class FtpConnector extends AbstractConnector
     public static final String PROPERTY_OUTPUT_PATTERN = "outputPattern"; // outbound only
     public static final String PROPERTY_PASSIVE_MODE = "passive";
     public static final String PROPERTY_BINARY_TRANSFER = "binary";
+    public static final String ASYNCHRONOUS_RECONNECTION_ERROR_MESSAGE = "FTP Connector doesn't support asynchronous retry policies.";
 
     // message properties
     public static final String PROPERTY_FILENAME = "filename";
@@ -155,8 +160,7 @@ public class FtpConnector extends AbstractConnector
     }
 
     /**
-     * Setter for property 'connectionFactoryClass'. Should be an instance of
-     * {@link FtpConnectionFactory}.
+     * Setter for property 'connectionFactoryClass'. Should be an instance of {@link FtpConnectionFactory}.
      *
      * @param connectionFactoryClass Value to set for property 'connectionFactoryClass'.
      */
@@ -165,40 +169,50 @@ public class FtpConnector extends AbstractConnector
         this.connectionFactoryClass = connectionFactoryClass;
     }
 
-    public FTPClient getFtp(EndpointURI uri) throws Exception
+    public FTPClient getFtp(ImmutableEndpoint endpoint) throws ConnectException
     {
+        FTPClient client;
         if (logger.isDebugEnabled())
         {
-            logger.debug(">>> retrieving client for " + uri);
-        }
-        return (FTPClient) getFtpPool(uri).borrowObject();
-    }
-
-    public void releaseFtp(EndpointURI uri, FTPClient client) throws Exception
-    {
-        if (logger.isDebugEnabled())
-        {
-            logger.debug("<<< releasing client for " + uri);
-        }
-        if (dispatcherFactory.isCreateDispatcherPerRequest())
-        {
-            destroyFtp(uri, client);
-        }
-        else
-        {
-            getFtpPool(uri).returnObject(client);
-        }
-    }
-
-    public void destroyFtp(EndpointURI uri, FTPClient client) throws Exception
-    {
-        if (logger.isDebugEnabled())
-        {
-            logger.debug("<<< destroying client for " + uri);
+            logger.debug(">>> retrieving client for " + endpoint.getEndpointURI());
         }
         try
         {
-            getFtpPool(uri).invalidateObject(client);
+            client = (FTPClient) getFtpPool(endpoint).borrowObject();
+        }
+        catch(Exception e)
+        {
+            throw new ConnectException(e, this);
+        }
+
+        return client;
+    }
+
+    public void releaseFtp(ImmutableEndpoint endpoint, FTPClient client) throws Exception
+    {
+        if (logger.isDebugEnabled())
+        {
+            logger.debug("<<< releasing client for " + endpoint.getEndpointURI());
+        }
+        if (dispatcherFactory.isCreateDispatcherPerRequest())
+        {
+            destroyFtp(endpoint, client);
+        }
+        else
+        {
+            getFtpPool(endpoint).returnObject(client);
+        }
+    }
+
+    public void destroyFtp(ImmutableEndpoint endpoint, FTPClient client) throws Exception
+    {
+        if (logger.isDebugEnabled())
+        {
+            logger.debug("<<< destroying client for " + endpoint.getEndpointURI());
+        }
+        try
+        {
+            getFtpPool(endpoint).invalidateObject(client);
         }
         catch (Exception e)
         {
@@ -207,8 +221,9 @@ public class FtpConnector extends AbstractConnector
         }
     }
 
-    protected synchronized ObjectPool getFtpPool(EndpointURI uri)
+    protected synchronized ObjectPool getFtpPool(ImmutableEndpoint endpoint)
     {
+        EndpointURI uri = endpoint.getEndpointURI();
         if (logger.isDebugEnabled())
         {
             logger.debug("=== get pool for " + uri);
@@ -223,6 +238,7 @@ public class FtpConnector extends AbstractConnector
                         (FtpConnectionFactory) ClassUtils.instanciateClass(getConnectionFactoryClass(),
                                                                             new Object[] {uri}, getClass());
                 connectionFactory.setConnectionTimeout(connectionTimeout);
+                connectionFactory.setResponseTimeout(endpoint.getResponseTimeout());
                 GenericObjectPool genericPool = createPool(connectionFactory);
                 pools.put(key, genericPool);
                 pool = genericPool;
@@ -543,57 +559,57 @@ public class FtpConnector extends AbstractConnector
      *         does not support streaming
      */
     @Override
-    public OutputStream getOutputStream(OutboundEndpoint endpoint, MuleEvent event) throws MuleException
+    public OutputStream getOutputStream(final OutboundEndpoint endpoint, MuleEvent event) throws MuleException
     {
         try
         {
             final EndpointURI uri = endpoint.getEndpointURI();
             String filename = getFilename(endpoint, event.getMessage());
 
-            final FTPClient client;
-            try
+            final FTPClient[] client = {null};
+            RetryCallback callbackReconnection = new RetryCallback()
             {
-                client = this.createFtpClient(endpoint);
-            }
-            catch (Exception e)
-            {
-                throw new ConnectException(e, this);
-            }
 
-            try
-            {
-                OutputStream out = client.storeFileStream(filename);
-                if (out == null)
+                @Override
+                public void doWork(RetryContext context) throws Exception
                 {
-                    throw new IOException("FTP operation failed: " + client.getReplyString());
+                    client[0] = createFtpClient(endpoint);
                 }
 
-                return new CallbackOutputStream(out,
-                        new CallbackOutputStream.Callback()
-                        {
-                            public void onClose() throws Exception
-                            {
-                                try
-                                {
-                                    if (!client.completePendingCommand())
-                                    {
-                                        client.logout();
-                                        client.disconnect();
-                                        throw new IOException("FTP Stream failed to complete pending request");
-                                    }
-                                }
-                                finally
-                                {
-                                    releaseFtp(uri, client);
-                                }
-                            }
-                        });
-            }
-            catch (Exception e)
+                @Override
+                public String getWorkDescription()
+                {
+                    return getConnectionDescription();
+                }
+
+                @Override
+                public Connector getWorkOwner()
+                {
+                    return FtpConnector.this;
+                }
+            };
+            if (getRetryPolicyTemplate().isSynchronous())
             {
-                logger.debug("Error getting output stream: ", e);
-                releaseFtp(uri, client);
-                throw e;
+                try
+                {
+                    RetryContext context = getRetryPolicyTemplate().execute(callbackReconnection, muleContext.getWorkManager());
+                    if (client[0] != null)
+                    {
+                        return storeFileStream(client[0], filename, endpoint);
+                    }
+                    else
+                    {
+                        throw new ConnectException(context.getLastFailure(), this);
+                    }
+                }
+                catch (RetryPolicyExhaustedException retryPolicyExhaustedException)
+                {
+                    throw retryPolicyExhaustedException;
+                }
+            }
+            else
+            {
+                throw new IllegalArgumentException(ASYNCHRONOUS_RECONNECTION_ERROR_MESSAGE);
             }
         }
         catch (ConnectException ce)
@@ -607,6 +623,45 @@ public class FtpConnector extends AbstractConnector
         }
     }
 
+    private OutputStream storeFileStream(final FTPClient client, String filename, final ImmutableEndpoint endpoint) throws Exception
+    {
+        try
+        {
+            OutputStream out = client.storeFileStream(filename);
+            if (out == null)
+            {
+                throw new IOException("FTP operation failed: " + client.getReplyString());
+            }
+
+            return new CallbackOutputStream(out,
+                                            new CallbackOutputStream.Callback()
+                                            {
+                                                public void onClose() throws Exception
+                                                {
+                                                    try
+                                                    {
+                                                        if (!client.completePendingCommand())
+                                                        {
+                                                            client.logout();
+                                                            client.disconnect();
+                                                            throw new IOException("FTP Stream failed to complete pending request");
+                                                        }
+                                                    }
+                                                    finally
+                                                    {
+                                                        releaseFtp(endpoint, client);
+                                                    }
+                                                }
+                                            });
+        }
+        catch (Exception e)
+        {
+            logger.debug("Error getting output stream: ", e);
+            releaseFtp(endpoint, client);
+            throw e;
+        }
+    }
+
     private String getFilename(ImmutableEndpoint endpoint, MuleMessage message) throws IOException
     {
         String filename = message.getOutboundProperty(FtpConnector.PROPERTY_FILENAME);
@@ -642,7 +697,7 @@ public class FtpConnector extends AbstractConnector
     protected FTPClient createFtpClient(ImmutableEndpoint endpoint) throws Exception
     {
         EndpointURI uri = endpoint.getEndpointURI();
-        FTPClient client = this.getFtp(uri);
+        FTPClient client = this.getFtp(endpoint);
         client.setDataTimeout(endpoint.getResponseTimeout());
 
         this.enterActiveOrPassiveMode(client, endpoint);
@@ -651,31 +706,38 @@ public class FtpConnector extends AbstractConnector
         String path = uri.getPath();
 
         // only change directory if one was configured
-        if (StringUtils.isNotBlank(path))
+        if (StringUtils.isBlank(path))
         {
-            // MULE-2400: if the path begins with '~' we must strip the first '/' to make things
-            // work with FTPClient
-            if ((path.length() >= 2) && (path.charAt(1) == '~'))
-            {
-                path = path.substring(1);
-            }
+            return client;
+        }
 
-            //Checking if it is a file or a directory
-            boolean isFile = this.isFile(endpoint, client);
-            if (!isFile && !client.changeWorkingDirectory(path))
+        // MULE-2400: if the path begins with '~' we must strip the first '/' to make things
+        // work with FTPClient
+        if ((path.length() >= 2) && (path.charAt(1) == '~'))
+        {
+            path = path.substring(1);
+        }
+
+        //Checking if it is a file or a directory
+        boolean isFile = this.isFile(endpoint, client);
+        if (!isFile && !client.changeWorkingDirectory(path))
+        {
+            throw new IOException(MessageFormat.format("Failed to change working directory to {0}. Ftp error: {1}",
+                    path, client.getReplyCode()));
+        }
+        else if (isFile)
+        {
+            // Changing the working directory to the parent folder, it should be better if
+            // the ftpClient API would provide a way to retrieve the parent folder
+            FTPFile[] listFiles = client.listFiles(path);
+            // Files could have been consumed, e.g. by an external FTP Client
+            if (listFiles.length == 1)
             {
-                throw new IOException(MessageFormat.format("Failed to change working directory to {0}. Ftp error: {1}",
-                                                           path, client.getReplyCode()));
-            }
-            else if (isFile)
-            {
-                // Changing the working directory to the parent folder, it should be better if
-                // the ftpClient API would provide a way to retrieve the parent folder
-                FTPFile[] listFiles = client.listFiles(path);
                 String directory = path.replaceAll(listFiles[0].getName(), "");
                 client.changeWorkingDirectory(directory);
             }
         }
+
         return client;
     }
 
